
#ifndef LIBDIALOGUTIL_EXPORT_H
#define LIBDIALOGUTIL_EXPORT_H

#ifdef LIBDIALOGUTIL_STATIC_DEFINE
#  define LIBDIALOGUTIL_EXPORT
#  define LIBDIALOGUTIL_NO_EXPORT
#else
#  ifndef LIBDIALOGUTIL_EXPORT
#    ifdef dialogutil_EXPORTS
        /* We are building this library */
#      define LIBDIALOGUTIL_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define LIBDIALOGUTIL_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef LIBDIALOGUTIL_NO_EXPORT
#    define LIBDIALOGUTIL_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef LIBDIALOGUTIL_DEPRECATED
#  define LIBDIALOGUTIL_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef LIBDIALOGUTIL_DEPRECATED_EXPORT
#  define LIBDIALOGUTIL_DEPRECATED_EXPORT LIBDIALOGUTIL_EXPORT LIBDIALOGUTIL_DEPRECATED
#endif

#ifndef LIBDIALOGUTIL_DEPRECATED_NO_EXPORT
#  define LIBDIALOGUTIL_DEPRECATED_NO_EXPORT LIBDIALOGUTIL_NO_EXPORT LIBDIALOGUTIL_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef LIBDIALOGUTIL_NO_DEPRECATED
#    define LIBDIALOGUTIL_NO_DEPRECATED
#  endif
#endif

#endif /* LIBDIALOGUTIL_EXPORT_H */
